package model;

import java.awt.Color;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Scanner;

public class GameBoard {
    private int width, height;
    private Item origTable[][];
    public  Item table[][];
    private Player[] players;
    public final int scale, numPlayers;
    public final int playerSpeed = 10;
    public final int turnTolerance = playerSpeed;
    private int numFoods, numPower, numFoodsRemain, numPowerRemain;

    public GameBoard(int numPlayers, int scale){
        this.players = new Player[numPlayers];
        this.numPlayers = numPlayers;
        this.scale = scale;
        loadGame();
        initGame();
    }
    
    private void loadGame(){
        ArrayList<String> lines = new ArrayList<>();
        int maxWidth = 0;
        try {
            InputStream is = getClass().getResourceAsStream("../res/pacman.txt");
            Scanner s = new Scanner(is);
            while (s.hasNextLine()){
                String line = s.nextLine();
                lines.add(line);
                if (line.length() > maxWidth) maxWidth = line.length();
            }
            is.close();
        } catch (IOException ex) {}
        width = maxWidth;
        height = lines.size();
        numFoods = 0;
        numPower = 0;
        
        if (width > 0 && height > 0){
            origTable = new Item[height][width];
            table = new Item[height][width];
            for (int y = 0; y < height; y++){
                String line = lines.get(y);
                for (int x = 0; x < line.length(); x++){
                    char c = line.charAt(x);
                    if (c == ' '){
                        origTable[y][x] = Item.EMPTY;
                    } else if (c == '.'){
                        origTable[y][x] = Item.FOOD;
                        numFoods++;
                    } else if (c == 'o'){
                        origTable[y][x] = Item.POWER;
                        numPower++;
                    } else if (c == 'X'){
                        origTable[y][x] = Item.WALL;                        
                    } else if (c == '1'){
                        players[0] = new Player(x, y, scale, Color.YELLOW, playerSpeed);
                    } else if (c == '2'){
                        players[1] = new Player(x, y, scale, Color.PINK, playerSpeed);
                    } else {
                        origTable[y][x] = Item.EMPTY;
                    }
                }
            }
        }
    }
    
    public void initGame(){
        for (int y = 0; y < height; y++){
            for (int x = 0; x < width; x++){
                table[y][x] = origTable[y][x];
            }
        }
        numFoodsRemain = numFoods;
        numPowerRemain = numPower;
        for (int i = 0; i < numPlayers; i++) {
            players[i].reset();
        }
    }

    public int getWidth() { return width; }

    public int getHeight() { return height; }

    public Player getPlayer(int id){ return players[id]; }
    
    public void changePlayerDirection(int player, Direction d){
        Player p = players[player];
        boolean sameAxis = (p.moveDir.isHorizontal == d.isHorizontal && p.moveDir.isVertical == d.isVertical);
        if (sameAxis || p.moveDir == Direction.STAY){
            // leave or reverse the direction, or start the staying pacman
            p.nextDir = p.moveDir = p.faceDir = d;
        } else {
            // Turn pacman
            p.nextDir = d;
        }
    }
    
    public void movePlayers() {
        for (int i = 0; i < numPlayers; i++){
            Player p = players[i];
            Direction m = p.moveDir;
            Direction n = p.nextDir;
            if (m == Direction.STAY) continue;
            int offsX = 0, offsY = 0;
            int predX = p.smoothPos.x + m.x * playerSpeed;
            int predY = p.smoothPos.y + m.y * playerSpeed;

            boolean hasToTurn = (m.isHorizontal == n.isVertical);
            boolean canTurnFromDown = (p.smoothPos.y - p.scaledPos.y <= turnTolerance) && (table[p.pos.y][p.pos.x + n.x] != Item.WALL);
            boolean canTurnFromRight = (p.smoothPos.x - p.scaledPos.x <= turnTolerance) && (table[p.pos.y + n.y][p.pos.x] != Item.WALL);
            boolean canTurnFromUp = (p.scaledPos.y + scale - p.smoothPos.y <= turnTolerance) && (table[p.pos.y + 1][p.pos.x + n.x] != Item.WALL);
            boolean canTurnFromLeft = (p.scaledPos.x + scale - p.smoothPos.x <= turnTolerance) && (table[p.pos.y + n.y][p.pos.x + 1] != Item.WALL);

            if (hasToTurn){
                hasToTurn = false;
                if (m == Direction.RIGHT && canTurnFromLeft){
                    hasToTurn = true;
                    p.setSmoothX(p.scaledPos.x + scale);
                } else if (m == Direction.LEFT && canTurnFromRight){
                    hasToTurn = true;
                    p.setSmoothX(p.scaledPos.x);
                } else if (m == Direction.DOWN && canTurnFromUp){
                    hasToTurn = true;
                    p.setSmoothY(p.scaledPos.y + scale);
                } else if (m == Direction.UP && canTurnFromDown){
                    hasToTurn = true;
                    p.setSmoothY(p.scaledPos.y);
                }
                if (hasToTurn) {
                    m = p.faceDir = p.moveDir = p.nextDir;
                    tryEat(p.pos.x, p.pos.y);
                }
            }
            
            if (m == Direction.RIGHT){
                int indRight = (predX + scale - 1) / scale;
                int distRight = (indRight - 1) * scale - p.smoothPos.x;
                boolean canMoveRight = table[p.pos.y][indRight] != Item.WALL;
                offsX = (canMoveRight) ? playerSpeed : distRight;
            } else if (m == Direction.LEFT){
                int indLeft = predX / scale;
                int distLeft = p.smoothPos.x - p.scaledPos.x;
                boolean canMoveLeft = table[p.pos.y][indLeft] != Item.WALL;
                offsX = (canMoveLeft) ? -playerSpeed : -distLeft;
            } else if (m == Direction.UP){
                int indUp = predY / scale;
                int distUp = p.smoothPos.y - p.scaledPos.y;
                boolean canMoveUp = table[indUp][p.pos.x] != Item.WALL;
                offsY = (canMoveUp) ? -playerSpeed : -distUp;
            } else if (m == Direction.DOWN){
                int indDown = (predY + scale - 1) / scale;
                int distDown = (indDown - 1) * scale - p.smoothPos.y;
                boolean canMoveDown = table[indDown][p.pos.x] != Item.WALL;
                offsY = (canMoveDown) ? playerSpeed : distDown;
            }
            
            if (offsX != 0 || offsY != 0){
                p.animateMouth();
                p.setSmoothCoord(p.smoothPos.x + offsX, p.smoothPos.y + offsY);
                tryEat(p.pos.x, p.pos.y);
            } else {
                p.moveDir = Direction.STAY;
            }
        }
        if (numFoodsRemain == 0){
            initGame();
        }
    }
    
    private void tryEat(int x, int y){
        if (table[y][x] == Item.FOOD){
            numFoodsRemain--;
            table[y][x] = Item.EMPTY;
        } else if (table[y][x] == Item.POWER){
            numPowerRemain--;
            table[y][x] = Item.EMPTY;
        }
    }
}
