package model;

import java.awt.Color;
import java.awt.Point;
import java.io.Serializable;

public class Player {
    public final Color color;
    public final Point pos, smoothPos, scaledPos;
    public Direction moveDir, faceDir, nextDir;
    public int playerSpeed;
    
    private final Point startPos;
    private final int mouthSpeed = 10;
    private final int maxOpenMouth = 40;
    private final int scale;
    private int mouth;
    private boolean mouthOpens;

    public Player(int x, int y, int scale, Color color, int speed) {
        pos = new Point(x, y);
        startPos = new Point(x, y);
        scaledPos = new Point(x * scale, y * scale);
        smoothPos = new Point(scaledPos);
        this.mouth = maxOpenMouth;
        this.mouthOpens = true;
        this.moveDir = Direction.STAY;
        this.faceDir = Direction.RIGHT;
        this.nextDir = Direction.STAY;
        this.color = color;
        this.scale = scale;
        this.playerSpeed = speed;
    }
    
    public void animateMouth(){
        if (mouthOpens){
            mouth += mouthSpeed;
        } else {
            mouth -= mouthSpeed;
        }
        if (mouth >= maxOpenMouth) mouthOpens = false;
        if (mouth <= mouthSpeed) mouthOpens = true;
    }
    
    public int getMouth() {
        return mouth;
    }
    
    public void setSmoothX(int sx){
        smoothPos.x = sx;
        pos.x = sx / scale;
        scaledPos.x = pos.x * scale;
    }
    
    public void setSmoothY(int sy){
        smoothPos.y = sy;
        pos.y = sy / scale;
        scaledPos.y = pos.y * scale;
    }
    
    public void setSmoothCoord(int sx, int sy){
        setSmoothX(sx);
        setSmoothY(sy);
    }

    public void setNextDir(Direction d){
        nextDir = d;
    }
    
    public void reset(){
        pos.setLocation(startPos);
        scaledPos.setLocation(pos.x * scale, pos.y * scale);
        smoothPos.setLocation(scaledPos);
        moveDir = Direction.STAY;
        faceDir = Direction.RIGHT;
        nextDir = Direction.STAY;
        mouth = maxOpenMouth;
        mouthOpens = true;
    }
}